import {Component, Element, Prop, h, Host} from '@stencil/core';
import {CtaLinkButtonVariants} from '../../utils/utils';

@Component({
  tag: 'hy-banner',
  styleUrl: 'hy-banner.scss',
  shadow: false,
})
export class HyBanner {
  @Prop() textTitle?: string;
  @Prop() textDescription: string = null;
  @Prop() url?: string;
  @Prop() scLabel?: string;
  @Prop() urlTitle?: string;
  @Prop() headerstyle: string = 'common';
  @Prop() isExternal: boolean = false;
  @Element() el: HTMLElement;

  componentDidLoad() {
    let hyMainDiv = this.el.closest('.hy-main');
    if (hyMainDiv) {
      if (!hyMainDiv.classList.contains('with-sidebar')) {
        this.headerstyle = 'large';
      }
    }
  }

  render() {
    const hyBannerClass = ['hy-banner', `hy-banner__${this.headerstyle}`, this.headerstyle].join(' ');

    const hyBannerWrapperClass = ['hy-banner__info-container'].join(' ');
    const hyBannerTitle = [
      'hy-banner__info-container__title',
      `hy-banner__info-container__title__${this.headerstyle}`,
      this.headerstyle,
    ].join(' ');
    const hyBannerDescription = [
      'hy-banner__info-container__description',
      `hy-banner__info-container__description__${this.headerstyle}`,
      this.headerstyle,
    ].join(' ');

    const variant = CtaLinkButtonVariants.onBlueBkgd;

    return (
      <Host>
        <hy-box pt="1.25, 1.25, 1.5, 2.5" />
        <div class={hyBannerClass}>
          <hy-main-content-wrapper>
            <div class={hyBannerWrapperClass}>
              <div class={hyBannerTitle}>{this.textTitle}</div>
              <div class={hyBannerDescription}>{this.textDescription}</div>
              {this.url && (
                <div class="hy-banner__info-container__link">
                  <hy-cta-button
                    link-content={this.urlTitle}
                    sc-label={this.scLabel}
                    url={this.url}
                    is-external={this.isExternal}
                    variant={variant}
                  />
                </div>
              )}
            </div>
          </hy-main-content-wrapper>
        </div>
        <hy-box mb="1.75, 1.75, 2, 2.5" />
      </Host>
    );
  }
}
