export interface KeyHighlightValue {
  heading: string;
  description: string;
}

import {Component, ComponentInterface, Watch, Prop, h, Host, Element} from '@stencil/core';
import {KeyHighlightVariants} from '../../utils/utils';

@Component({
  tag: 'hy-key-highlight-group',
  styleUrl: 'hy-key-highlight-group.scss',
  shadow: true,
})
export class HyKeyHighlightGroup implements ComponentInterface {
  @Element() el: HTMLElement;
  @Prop() variant: KeyHighlightVariants = KeyHighlightVariants.default;
  private _dataItems: KeyHighlightValue[];
  @Prop() dataItems: KeyHighlightValue[] | string;
  @Prop() headerstyle: string = 'common';

  @Watch('dataItems')
  arrayDataWatcher(newValue: KeyHighlightValue[] | string) {
    if (typeof newValue === 'string') {
      this._dataItems = JSON.parse(newValue);
    } else {
      this._dataItems = newValue;
    }
  }
  componentWillLoad() {
    this.arrayDataWatcher(this.dataItems);
  }

  componentDidLoad() {
    let hyMainDiv = this.el.closest('.hy-main');
    if (hyMainDiv) {
      if (!hyMainDiv.classList.contains('with-sidebar')) {
        this.headerstyle = 'large';
      }
    }
  }

  render() {
    const classAttributes = [
      this.variant,
      'hy-key-highlight-group',
      `hy-key-highlight-group__${this.headerstyle}`,
    ].join(' ');

    const dataItemCount = this._dataItems.length;
    const classItem = dataItemCount % 4 == 0 ? 'box-4' : 'box';

    return (
      <Host>
        <div class={classAttributes}>
          {this._dataItems.map((x) => {
            return (
              <hy-key-highlight
                class={classItem}
                variant={this.variant}
                heading={x.heading}
                description={x.description}
              />
            );
          })}
        </div>
      </Host>
    );
  }
}
