export interface CtaLinkValue {
  heading: string;
  ariaLabel: string;
  url: string;
  isExternal: boolean;
}

import {Component, ComponentInterface, Element, h, Prop, Watch} from '@stencil/core';
import {CtaLinkVariants} from '../../utils/utils';

@Component({
  tag: 'hy-link-list',
  styleUrl: 'hy-link-list.scss',
  shadow: true,
})
export class HyLinkList implements ComponentInterface {
  @Element() el: HTMLElement;
  @Prop() headerstyle: string = 'default';
  private _dataItems: CtaLinkValue[];
  @Prop() dataItems: CtaLinkValue[] | string;
  @Prop() listHeading: string = '';

  @Watch('dataItems')
  arrayDataWatcher(newValue: CtaLinkValue[] | string) {
    if (typeof newValue === 'string') {
      this._dataItems = JSON.parse(newValue);
    } else {
      this._dataItems = newValue;
    }
  }
  componentWillLoad() {
    this.arrayDataWatcher(this.dataItems);
  }

  componentDidLoad() {
    let hyMainDiv = this.el.closest('.hy-main');
    if (hyMainDiv) {
      if (!hyMainDiv.classList.contains('with-sidebar')) {
        this.headerstyle = 'large';
      }
    }
  }

  render() {
    const classAttributes = ['hy-link-list', `hy-link-list__${this.headerstyle}`].join(' ');
    const linkVariant = CtaLinkVariants.linkList;

    return [
      <div class={classAttributes}>
        {this.listHeading.length > 0 && <div class={'list-title'}>{this.listHeading}</div>}
        {this._dataItems.map((x) => {
          return (
            <hy-cta-link
              variant={linkVariant}
              link-content={x.heading}
              sc-label={x.ariaLabel}
              url={x.url}
              is-external={x.isExternal}
            />
          );
        })}
      </div>,
    ];
  }
}
